//	I2C.c - IH I2C Driver for Bus Master, Standard Speed only.
#include <reg51xd2.h>
#include "i2c.h"

static unsigned char 	I2C_NackCount = 0;


/************************** LOCAL *****************************/
/*
*	I2C_Delay - Variable delay = (2.7 + 0.8i) us in calling environment
*
*/
static void		I2C_Delay(unsigned char i){
	while(--i) ;		/* DJNZ */
	}


/************************* GLOBAL *****************************/
/*
*	I2C_Start - Transmit the start condition
*/
void	I2C_Start(void){

	SDA = 0;	I2C_Delay(3);
	SCL = 0;	I2C_Delay(1);
	I2C_NackCount = 0;					/* RESET I2C_NackCount */
	}

/*
*	I2C_RepeatStart - Re-Transmit the start condition
*
*	Does not reset I2C_NackCount
*/

void	I2C_RepeatStart(void){

	/* AT ENTRY SCL = LOW */

	/* GET BUS IN RIGHT CONDITION */
	SDA = 1;
	SCL = 1;	I2C_Delay(3);

	/* ISSUE START CONDITION */
	SDA = 0;	I2C_Delay(3);
	SCL = 0;	I2C_Delay(2);
	}

/*
*	I2C_Stop - Transmit the stop condition
*/

unsigned char	I2C_Stop(void){
	/* AT ENTRY SCL = LOW */
	SDA = 0;

	SCL = 1;	I2C_Delay(3);
	SDA = 1;	I2C_Delay(2);

	return I2C_NackCount;
	}

/*
*	I2C_Transmit - Transmit a byte to a slave receiver
*
*	Returns ACK status
*/
unsigned char	I2C_Transmit(unsigned char B){
	unsigned char data		i;

	/* AT ENTRY SCL = LOW */

	/* TRANSMIT 8 BITS */
	for	(i = 8 ; i ; i--){
		if ((B & 0x80) == 0x80)
			SDA = 1;				/* ASSERT DATA */
		else
			SDA = 0;
		SCL = 1;				/* CLOCK HIGH */
		B <<= 1;				/* SHIFT BYTE */
		I2C_Delay(1);			/* CLOCK HIGH DELAY */
		SCL = 0;				/* CLOCK LOW */
		I2C_Delay(1);			/* CLOCK LOW DELAY */
		}

	/* GET ACK */
	SDA = 1;					/* RELEASE DATA LINE */
	SCL = 1;					/* CLOCK HIGH */
	I2C_Delay(1);				/* CLOCK HIGH DELAY */
	i = SDA;					/* GET ACK */
	SCL = 0;					/* CLOCK LOW */
	I2C_Delay(1);				/* CLOCK LOW DELAY */

	/* AT EXIT SCL = LOW, SDA = HIGH */
	I2C_NackCount += i;			/* INCREMENT NACKS */
	return i;
	}

/*
*	I2C_Receive - Receive a byte from a slave transmitter, 
*				and send parameter a as ACK.
*/
unsigned char	I2C_Receive(unsigned char a){
	unsigned char data		i, B = 0;

	/* AT ENTRY SCL = LOW */
	SDA = 1;					/* RELEASE SDA */

	/* RECEIVE 8 BITS */
	for	(i = 8 ; i ; i--){
		SCL = 1;				/* CLOCK HIGH */
		B <<= 1;				/* SHIFT BYTE (SOME DELAY HERE) */
		if	(SDA)				/* ACQUIRE DATA */
			B |= 0x01;
 		I2C_Delay(1);			/* CLOCK HIGH DELAY */
		SCL = 0;				/* CLOCK LOW */
		I2C_Delay(1);			/* CLOCK LOW DELAY */
		}

	/* SEND [N]ACK */
	if	(!a){
		SDA = 0;					/* DATA LOW */
		SCL = 1;					/* CLOCK HIGH */
		I2C_Delay(3);				/* CLOCK HIGH DELAY */
		SCL = 0;					/* CLOCK LOW */
		SDA = 1;					/* DATA HIGH */
		I2C_Delay(1);				/* CLOCK LOW DELAY */
		}
	else{
		SCL = 1;					/* CLOCK HIGH */
		I2C_Delay(3);				/* CLOCK HIGH DELAY */
		SCL = 0;					/* CLOCK LOW */
		I2C_Delay(1);				/* CLOCK LOW DELAY (tLOW) */
		}

	return B;
}
